pre.sh_sourceCode {
  background-color: #f3f3f3;
  display: inline-block;
  color: #121b28;
  font-weight: normal;
  font-style: normal;
  padding: 5px;
  -moz-border-radius: 10px;
  border-radius: 10px;
}

pre.sh_sourceCode .sh_keyword {
  color: #0a7f6d;
  font-weight: normal;
  font-style: normal;
}

pre.sh_sourceCode .sh_type {
  color: #c42638;
  font-weight: normal;
  font-style: normal;
}

pre.sh_sourceCode .sh_string {
  color: #2b83ba;
  font-weight: normal;
  font-style: normal;
}

pre.sh_sourceCode .sh_regexp {
  color: #2b83ba;
  font-weight: normal;
  font-style: normal;
}

pre.sh_sourceCode .sh_specialchar {
  color: #a764cb;
  font-weight: normal;
  font-style: normal;
}

pre.sh_sourceCode .sh_comment {
  color: #444444;
  font-weight: normal;
  font-style: normal;
}

pre.sh_sourceCode .sh_number {
  color: sienna;
  font-weight: normal;
  font-style: normal;
}

pre.sh_sourceCode .sh_preproc {
  color: #620ac6;
  font-weight: normal;
  font-style: normal;
}

pre.sh_sourceCode .sh_symbol {
  color: purple;
  font-weight: normal;
  font-style: normal;
}

pre.sh_sourceCode .sh_function {
  color: #000000;
  font-weight: bold;
  font-style: normal;
}

pre.sh_sourceCode .sh_cbracket {
  color: #342D7E;
  font-weight: normal;
  font-style: normal;
}

pre.sh_sourceCode .sh_url {
  color: #2b83ba;
  font-weight: normal;
  font-style: normal;
}

pre.sh_sourceCode .sh_date {
  color: #0a7f6d;
  font-weight: bold;
  font-style: normal;
}

pre.sh_sourceCode .sh_time {
  color: #0a7f6d;
  font-weight: bold;
  font-style: normal;
}

pre.sh_sourceCode .sh_file {
  color: #0a7f6d;
  font-weight: bold;
  font-style: normal;
}

pre.sh_sourceCode .sh_ip {
  color: #2b83ba;
  font-weight: normal;
  font-style: normal;
}

pre.sh_sourceCode .sh_name {
  color: #2b83ba;
  font-weight: normal;
  font-style: normal;
}

pre.sh_sourceCode .sh_variable {
  color: darkblue;
  font-weight: normal;
  font-style: normal;
}

pre.sh_sourceCode .sh_oldfile {
  color: #a764cb;
  font-weight: normal;
  font-style: normal;
}

pre.sh_sourceCode .sh_newfile {
  color: #2b83ba;
  font-weight: normal;
  font-style: normal;
}

pre.sh_sourceCode .sh_difflines {
  color: #0a7f6d;
  font-weight: bold;
  font-style: normal;
}

pre.sh_sourceCode .sh_selector {
  color: #e12f76;
  font-weight: bold;
  font-style: normal;
}

pre.sh_sourceCode .sh_property {
  color: #0a7f6d;
  font-weight: bold;
  font-style: normal;
}

pre.sh_sourceCode .sh_value {
  color: #2b83ba;
  font-weight: normal;
  font-style: normal;
}

